class GeneralFunctionsQueryBuilder {
  constructor(elements) {
    this.elements = elements;
  }

  toggleClass(elements, className, action){
    elements.forEach(element => {
      element.classList[action](className);
    });
  }

  closeQueryBuilderModal(closeInstance) {
    let instance = M.Modal.getInstance(closeInstance);
    instance.close();
  }
  
  showPositionalAttrArea() {
    // this.elements.positionalAttrArea.classList.remove('hide');
    // this.elements.structuralAttrArea.classList.add('hide');
    this.toggleClass([this.elements.positionalAttrArea], 'hide', 'remove');
    this.toggleClass([this.elements.structuralAttrArea], 'hide', 'add');
    this.wordBuilder();

    this.elements.tokenQueryFilled = false;

    window.location.href = '#token-builder-content';
  }

  showStructuralAttrArea() {
    this.elements.positionalAttrArea.classList.add('hide');
    this.elements.structuralAttrArea.classList.remove('hide');
  }

  wordBuilder() {
    this.hideEverything();
    this.elements.wordInput.value = '';
    this.elements.wordBuilder.classList.remove('hide');
    this.elements.inputOptions.classList.remove('hide');
    this.elements.incidenceModifiersButton.classList.remove('hide');
    this.elements.conditionContainer.classList.remove('hide');
    this.elements.ignoreCaseCheckbox.classList.remove('hide');
    
    this.elements.incidenceModifiersButton.firstElementChild.classList.add('disabled');
    this.elements.or.classList.add('disabled');
    this.elements.and.classList.add('disabled');
    
    // Resets materialize select field to default value 
    let SelectInstance = M.FormSelect.getInstance(this.elements.positionalAttr);
    SelectInstance.input.value = 'word';
    this.elements.positionalAttr.value = 'word';

  }

  queryChipFactory(dataType, prettyQueryText, queryText) {
    this.elements.counter++;
    window.location.href = '#query-container';
    queryText = Utils.escape(queryText);
    prettyQueryText = Utils.escape(prettyQueryText);
    let queryChipElement = Utils.HTMLToElement(
      `
        <span class="chip query-component" data-type="${dataType}" data-query="${queryText}" draggable="true">
          ${prettyQueryText}
          <i class="material-icons close">close</i>
        </span>
      `
    );
    queryChipElement.addEventListener('click', () => {this.deleteAttr(queryChipElement);});
    queryChipElement.addEventListener('dragstart', (event) => {
      // selects all nodes without target class
      let queryChips = this.elements.yourQuery.querySelectorAll('.query-component');

      // Adds a target chip in front of all draggable childnodes
      setTimeout(() => {
        let targetChipElement = Utils.HTMLToElement('<span class="chip drop-target">Drop here</span>');
        for (let element of queryChips) {
          if (element === queryChipElement.nextSibling) {continue;}
          let targetChipClone = targetChipElement.cloneNode(true);
          if (element === queryChipElement) {
            // If the dragged element is not at the very end, a target chip is also inserted at the end
            if (queryChips[queryChips.length - 1] !== element) {
              queryChips[queryChips.length - 1].insertAdjacentElement('afterend', targetChipClone);
            }
          } else {
            element.insertAdjacentElement('beforebegin', targetChipClone);
          }
          targetChipClone.addEventListener('dragover', (event) => {
            event.preventDefault();
          });
          targetChipClone.addEventListener('dragenter', (event) => {
            event.preventDefault();
            event.target.style.borderStyle = 'solid dotted';
          });
          targetChipClone.addEventListener('dragleave', (event) => {
            event.preventDefault();
            event.target.style.borderStyle = 'hidden';
          });
          targetChipClone.addEventListener('drop', (event) => {
            let dropzone = event.target;
            dropzone.parentElement.replaceChild(queryChipElement, dropzone);
            this.queryPreviewBuilder();
          });
        }
      }, 0);
    });

    queryChipElement.addEventListener('dragend', (event) => {
      let targets = document.querySelectorAll('.drop-target');
      for (let target of targets) {
        target.remove();
      }
    });

    // Ensures that metadata is always at the end of the query:
    const lastChild = this.elements.yourQuery.lastChild;
    const isLastChildTextAnnotation = lastChild && lastChild.dataset.type === 'text-annotation';

    if (!isLastChildTextAnnotation) {
      this.elements.yourQuery.appendChild(queryChipElement);
    } else {
      this.elements.yourQuery.insertBefore(queryChipElement, lastChild);
    }

    this.elements.queryContainer.classList.remove('hide');
    this.queryPreviewBuilder();

    // Shows a hint about possible functions for editing the query at the first added element in the query
    if (this.elements.yourQuery.childNodes.length === 1) {
      app.flash('You can edit your query by deleting individual elements or moving them via drag and drop.');
    }
  }

  queryPreviewBuilder() {
    let queryPreview = document.querySelector('#query-preview');
    this.elements.yourQueryContent = [];
    for (let element of this.elements.yourQuery.childNodes) {
      let queryElement = element.dataset.query;
      if (queryElement !== undefined) {
        queryElement = Utils.escape(queryElement);
        this.elements.yourQueryContent.push(queryElement);
      }
    }

    let queryString = this.elements.yourQueryContent.join(' ');
    queryString += ';';

    queryPreview.innerHTML = queryString;
  }


  deleteAttr(attr) {
    this.elements.yourQuery.removeChild(attr);
    if (attr.dataset.type === "start-sentence") {
      this.elements.sentence.innerHTML = 'Sentence';
    } else if (attr.dataset.type === "start-entity" || attr.dataset.type === "start-empty-entity") {
      this.elements.entity.innerHTML = 'Entity';
    }
    this.elements.counter -= 1;
    if (this.elements.counter === 0) {
      this.elements.queryContainer.classList.add('hide');
    }
    this.queryPreviewBuilder();
  }

  insertQuery() {
    let extFormQuery= document.querySelector('#concordance-extension-form-query');
    this.elements.yourQueryContent = [];
    this.validateValue();
    if (this.elements.valueValidator) {
      for (let element of this.elements.yourQuery.childNodes) {
        let queryElement = element.dataset.query;
        if (queryElement !== 'undefined') {
          this.elements.yourQueryContent.push(queryElement);
        }
      }

      let queryString = this.elements.yourQueryContent.join(' ');
      queryString += ';';

      this.elements.concordanceQueryBuilder.classList.add('modal-close');
      extFormQuery.value = queryString;
    }
  }

  validateValue() {
    this.elements.valueValidator = true;
    let sentenceCounter = 0;
    let sentenceEndCounter = 0;
    let entityCounter = 0;
    let entityEndCounter = 0;
    for (let element of this.elements.yourQuery.childNodes) {
      if (element.dataset.type === 'start-sentence') {
        sentenceCounter += 1;
      }else if (element.dataset.type === 'end-sentence') {
        sentenceEndCounter += 1;
      }else if (element.dataset.type === 'start-entity' || element.dataset.type === 'start-empty-entity') {
        entityCounter += 1;
      }else if (element.dataset.type === 'end-entity') {
        entityEndCounter += 1;
      }
    }
    // Checks if the same number of opening and closing tags (entity and sentence) are present. Depending on what is missing, the corresponding error message is ejected
    if (sentenceCounter > sentenceEndCounter) {
      app.flash('Please add the closing sentence tag', 'error');
      this.elements.valueValidator = false;
    } else if (sentenceCounter < sentenceEndCounter) {
      app.flash('Please remove the closing sentence tag', 'error');
      this.elements.valueValidator = false;
    }
    if (entityCounter > entityEndCounter) {
      app.flash('Please add the closing entity tag', 'error');
      this.elements.valueValidator = false;
    } else if (entityCounter < entityEndCounter) {
      app.flash('Please remove the closing entity tag', 'error');
      this.elements.valueValidator = false;
    }
  }

  clearAll() {
    // Everything is reset.
    let instance = M.Tooltip.getInstance(this.elements.queryBuilderTutorialInfoIcon);

    this.hideEverything();
    this.elements.counter = 0;
    this.elements.concordanceQueryBuilder.classList.remove('modal-close');
    this.elements.positionalAttrArea.classList.add('hide');
    this.elements.structuralAttrArea.classList.add('hide');
    this.elements.yourQuery.innerHTML = '';
    this.elements.queryContainer.classList.add('hide');
    this.elements.entity.innerHTML = 'Entity';
    this.elements.sentence.innerHTML = 'Sentence';
    
    // If the Modal is open after 5 seconds for 5 seconds (with 'instance'), a message is displayed indicating that further information can be obtained via the question mark icon
    instance.tooltipEl.style.background = '#98ACD2';
    instance.tooltipEl.style.borderTop = 'solid 4px #0064A3';
    instance.tooltipEl.style.padding = '10px';
    instance.tooltipEl.style.color = 'black';

    setTimeout(() => {
      let modalInstance = M.Modal.getInstance(this.elements.concordanceQueryBuilder);
      if (modalInstance.isOpen) {
        instance.open();
        setTimeout(() => {
          instance.close();
        }, 5000);
      }
    }, 5000);
  }

  
  hideEverything() {
    let elementsToHide = ['wordBuilder', 'lemmaBuilder', 'ignoreCaseCheckbox', 'inputOptions', 'incidenceModifiersButton', 'conditionContainer', 'englishPosBuilder', 'germanPosBuilder', 'simplePosBuilder', 'entityBuilder', 'textAnnotationBuilder'];
    elementsToHide = elementsToHide.map(element => this.elements[element]);
    this.toggleClass(elementsToHide, 'hide', 'add');
  }

  tutorialIconHandler(id) {
    setTimeout(() => {
      window.location.href= id;
    }, 0);

  }
}
