class StructuralAttributeBuilderFunctionsQueryBuilder extends GeneralFunctionsQueryBuilder {
  constructor(elements) {
    super(elements);
    this.elements = elements;
  }

  addSentence() {
    this.hideEverything();
    if (this.elements.sentence.text === 'End Sentence') {
      this.queryChipFactory('end-sentence', 'Sentence End', '</s>');
      this.elements.sentence.innerHTML = 'Sentence';
    } else {
      this.queryChipFactory('start-sentence', 'Sentence Start', '<s>');
      this.elements.sentence.innerHTML = 'End Sentence';
    }
  }

  addEntity() {
    if (this.elements.entity.text === 'End Entity') {
      let queryText;
      if (this.elements.entityAnyType === false) {
        queryText = '</ent_type>'; 
      } else {
        queryText = '</ent>';
      }
      this.queryChipFactory('end-entity', 'Entity End', queryText);
      this.elements.entity.innerHTML = 'Entity';
    } else {
      this.hideEverything();
      this.elements.entityBuilder.classList.remove('hide');
      window.location.href = '#entity-builder';
    }
  }

  englishEntTypeHandler() {
    this.queryChipFactory('start-entity', 'Entity Type=' + this.elements.englishEntType.value, '<ent_type="' + this.elements.englishEntType.value + '">');
    this.elements.entity.innerHTML = 'End Entity';
    this.hideEverything();
    this.elements.entityAnyType = false;
    
    // Resets materialize select dropdown
    let SelectInstance = M.FormSelect.getInstance(this.elements.englishEntType);
    SelectInstance.input.value = 'English ent_type';
    this.elements.englishEntType.value = 'default';
  }   

  germanEntTypeHandler() {
    this.queryChipFactory('start-entity', 'Entity Type=' + this.elements.germanEntType.value, '<ent_type="' + this.elements.germanEntType.value + '">');
    this.elements.entity.innerHTML = 'End Entity';
    this.hideEverything();
    this.elements.entityAnyType = false;

    // Resets materialize select dropdown
    let SelectInstance = M.FormSelect.getInstance(this.elements.germanEntType);
    SelectInstance.input.value = 'German ent_type';
    this.elements.germanEntType.value = 'default';
  }

  emptyEntityButton() {
    this.queryChipFactory('start-empty-entity', 'Entity Start', '<ent>');
    this.elements.entity.innerHTML = 'End Entity';
    this.hideEverything();
    this.elements.entityAnyType = true;
  }

  addTextAnnotation() {
    this.hideEverything();
    this.elements.textAnnotationBuilder.classList.remove('hide');
    window.location.href = '#text-annotation-builder';

    // Resets materialize select dropdown
    let SelectInstance = M.FormSelect.getInstance(this.elements.textAnnotationOptions);
    SelectInstance.input.value = 'address';
    this.elements.textAnnotationOptions.value = 'address';
    this.elements.textAnnotationInput.value= '';
  }

  textAnnotationSubmitHandler() {
    let noValueMetadataMessage = document.querySelector('#no-value-metadata-message');
    if (this.elements.textAnnotationInput.value === '') {
      this.elements.textAnnotationSubmit.classList.add('red');
      noValueMetadataMessage.classList.remove('hide');
      setTimeout(() => {
        this.elements.textAnnotationSubmit.classList.remove('red');
      }, 500);
      setTimeout(() => {
        noValueMetadataMessage.classList.add('hide');
      }, 3000);
    } else {
      let queryText = `:: match.text_${this.elements.textAnnotationOptions.value}="${this.elements.textAnnotationInput.value}"`;
      this.queryChipFactory('text-annotation', `${this.elements.textAnnotationOptions.value}=${this.elements.textAnnotationInput.value}`, queryText);
      this.hideEverything();
    }
  }
}
